/**********************************************************************************
 * Proiectarea cu Microprocesoare
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <string.h>
#include "lcd.h"

#define STRAIGHT	1
#define CROSSOVER	2
#define ROLLOVER	3
#define NONE	   -1

//functie ce calculeaza tipul cablului introdus

int findType(int *v){ 
		int straight[8] = {1, 2, 3, 4, 5, 6, 7, 8};
		int cross[8] = {3, 6, 1, 4, 5, 2, 7, 8};
		int roll[8] = {8, 7, 6, 5, 4, 3, 2, 1};
		int i, ok = 0;
		//daca este straight
		for(i = 0; i < 8; i++){
			if(v[i] != straight[i]){
				ok = 1;//in cazul in care nepotrivire
				break;
			}
		}
		
		if(ok == 0)
			return STRAIGHT;
		
		ok = 0;
		
		for(i = 0; i < 8; i++){
			if(v[i] != cross[i]){
				ok = 1;
				break;
			}
		}

		if(ok == 0)
			return CROSSOVER;
		
		ok = 0;
		
		for(i = 0; i < 8; i++){
			if(v[i] != roll[i]){
				ok = 1;
				break;
			}
		}
		
		if(ok == 0)
			return ROLLOVER;
		
		return NONE;
}

int main()
{
	int i;
	LCD_init();
	LCD_writeInstruction(LCD_INSTR_clearDisplay);
	LCD_print("Starting...");
	_delay_ms(1000);

	//PORTA - sender
	//PORTD + B - receiver
	DDRA = 0xFF; //setez PORT A pe scriere
	DDRB = 0x00; //setez PORT C pe citire
	DDRD = 0x00; //setez PORT C pe citire
	
	int j, type;
	int v[8] = {0, 0, 0, 0, 0, 0, 0, 0};
	int z = 1;
	while(z == 1){
		PORTA = 0xFF; 
		_delay_ms(500);
		if(PIND == 0x00 || PINB == 0x00){ //daca nu este conectat cablul
			LCD_writeInstruction(LCD_INSTR_clearDisplay);
			LCD_print2("Please plug in", "a cable.");
		}
		else{
			for(i=7; i>=0; i--) {
		 		PORTA = 1<<i;
		 		if(PIND == 0x00 || PINB == 0x00){ //daca un fir nu ajunge nicaieri
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					LCD_print("Bad cable.");
					break;
				}
				_delay_ms(300);
				LCD_writeInstruction(LCD_INSTR_clearDisplay);
				LCD_print2("Checking cable,", "please wait...");
				//preia de pe portul D
				for(j = 7; j >= 4; j--){
						if(bit_is_set(PIND, j)){
							v[i] = j + 1;
						}
				}
				//preia de pe portul B
				PORTD = 0x00;
				for(j = 0; j <= 3; j++){
					    if(bit_is_set(PINB, j)){
							v[i] = j + 1;
						}
				}
				
				if(i == 0){
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
					//detecteaza tipul cablului
					type = findType(v);
					//afiseaza pe ecranul LCD-ului, tipul cablului detectat
					switch (type){
						case STRAIGHT:
							LCD_print2("This is a", "STRAIGHT cable.");
						break;
						case CROSSOVER:
							LCD_print2("This is a", "CROSSOVER cable.");
						break;
						case ROLLOVER:
							LCD_print2("This is a", "ROLLOVER cable.");
						break;
						default:
							LCD_print("UNDEFINED cable.");
							z = 10;
						break;
					}
					
					PORTA = 0x00;
					PORTD = 0x00;
					PORTB = 0x00;
					_delay_ms(3000);
					LCD_writeInstruction(LCD_INSTR_clearDisplay);
				}
			}
		}	
	}
	return 0;
}
